@@ -1,5 +1,5 @@
 /*
-% Copyright (C) 2003 - 2018 GraphicsMagick Group
+% Copyright (C) 2003 - 2019 GraphicsMagick Group
 % Copyright (C) 2002 ImageMagick Studio
 % Copyright 1991-1999 E. I. du Pont de Nemours and Company
 %
@@ -1151,7 +1151,7 @@ MagickExport MagickPassFail ModulateImage(Image *image,const char *modulate)
       (void) ModulateImagePixels(NULL,&param,image,image->colormap,
                                  (IndexPacket *) NULL,image->colors,
                                  &image->exception);
-      status=MagickMonitorFormatted(image->colors,image->colors+1,&image->exception,
+      status=MagickMonitorFormatted(image->colors,(size_t) image->colors+1,&image->exception,
                              progress_message,image->filename);
       status&=SyncImage(image);
     }
@@ -1258,7 +1258,7 @@ MagickExport MagickPassFail NegateImage(Image *image,const unsigned int grayscal
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   is_grayscale=image->is_grayscale;
-  if (*ImageGetClipMask(image))
+  if (*ImageGetClipMaskInlined(image))
     image->storage_class=DirectClass;
 
   if (image->storage_class == PseudoClass)
